@@ -1,4 +1,5 @@
 test_that("arimaSSM works properly",{
+tol<-1e-3
 s <- 12
 phis <- 0.99
 phi1 <- 0.0001
@@ -7,7 +8,7 @@ theta <-0.7
 out <- makeARIMA(phi,theta,NULL,SSinit="Ross")
 min(eigen(out$Pn)$value)
 out2<-SSMarima(phi,theta)
-expect_equivalent(out$Pn,out2$P1)
-expect_equivalent(out$T,out2$T)
-expect_equivalent(out$Z,c(out2$Z))
+expect_equal(out$Pn,out2$P1,tolerance=tol,check.attributes=FALSE)
+expect_equal(out$T,out2$T,tolerance=tol,check.attributes=FALSE)
+expect_equal(out$Z,c(out2$Z),tolerance=tol,check.attributes=FALSE)
 })
\ No newline at end of file
